@@ -126,13 +126,13 @@ class PyTDFunction(_function_base.Function):
     self.signatures = signatures
     self._signature_cache = {}
     self._return_types = {sig.pytd_sig.return_type for sig in signatures}
+    self._mutated_type_parameters = set()
     for sig in signatures:
       for param in sig.pytd_sig.params:
-        if param.mutated_type is not None:
-          self._has_mutable = True
-          break
-      else:
-        self._has_mutable = False
+        for params in sig.mutated_type_parameters[param]:
+          for template, value in params:
+            if template.type_param != value:
+              self._mutated_type_parameters.add(template.type_param.full_name)
     for sig in signatures:
       sig.function = self
       sig.name = self.name
@@ -321,11 +321,12 @@ class PyTDFunction(_function_base.Function):
     for v in values:
       if isinstance(v, _instance_base.SimpleValue):
         for name in v.instance_type_parameters:
-          mutations.append(
-              function.Mutation(
-                  v, name,
-                  self.ctx.convert.create_new_unknown(
-                      node, action="type_param_" + name)))
+          if name in self._mutated_type_parameters:
+            mutations.append(
+                function.Mutation(
+                    v, name,
+                    self.ctx.convert.create_new_unknown(
+                        node, action="type_param_" + name)))
     return mutations
 
   def _can_match_multiple(self, args):
@@ -365,9 +366,7 @@ class PyTDFunction(_function_base.Function):
             # the type of x should be Any, not int.
             view[arg] = arg.AddBinding(self.ctx.convert.unsolvable, [], node)
             break
-    if self._has_mutable:
-      # TODO(b/159055015): We only need to whack the type params that appear in
-      # a mutable parameter.
+    if self._mutated_type_parameters:
       mutations = self._get_mutation_to_unknown(
           node,
           (view[p].data if p in view else self.ctx.convert.unsolvable
@@ -495,6 +494,16 @@ class PyTDSignature(utils.ContextWeakrefMixin):
         for p in self.pytd_sig.params
     ]
     self.signature = function.Signature.from_pytd(ctx, name, pytd_sig)
+    self.mutated_type_parameters = {}
+    for p in self.pytd_sig.params:
+      try:
+        self.mutated_type_parameters[p] = self._collect_mutated_parameters(
+            p.type, p.mutated_type)
+      except ValueError as e:
+        log.error("Old: %s", pytd_utils.Print(p.type))
+        log.error("New: %s", pytd_utils.Print(p.mutated_type))
+        raise ValueError("Mutable parameters setting a type to a "
+                         "different base type is not allowed.") from e
 
   def _map_args(self, node, args):
     """Map the passed arguments to a name->binding dictionary.
@@ -605,6 +614,27 @@ class PyTDSignature(utils.ContextWeakrefMixin):
       log.info("param %d) %s: %s <=> %s", nr, p.name, p.type, arg_dict[p.name])
     return arg_dict, matches
 
+  def _paramspec_signature(self, callable_type, return_value, subst):
+    # Unpack the paramspec substitution we have created in the matcher.
+    # We should have two paramspec expressions, lhs and rhs, matching the
+    # higher-order function's args and return value respectively.
+    rhs = callable_type.args[0]
+    if isinstance(rhs, pytd.Concatenate):
+      r_pspec = rhs.paramspec
+      r_args = rhs.args
+    else:
+      r_pspec = rhs
+      r_args = ()
+    if r_pspec.full_name not in subst:
+      # TODO(b/217789659): Should this be an assertion failure?
+      return
+    ret = self.ctx.program.NewVariable()
+    for pspec_match in subst[r_pspec.full_name].data:
+      ret_sig = function.build_paramspec_signature(
+          pspec_match, r_args, return_value, self.ctx)
+      ret.AddBinding(_function_base.SimpleFunction(ret_sig, self.ctx))
+    return ret
+
   def _handle_paramspec(self, node, key, ret_map):
     """Construct a new function based on ParamSpec matching."""
     return_callable, subst = key
@@ -622,50 +652,9 @@ class PyTDSignature(utils.ContextWeakrefMixin):
           val.update_inner_type(k, typ)
     elif _isinstance(val, "TypeParameter") and val.full_name in subst:
       val = self.ctx.convert.merge_classes(subst[val.full_name].data)
-
-    # Unpack the paramspec substitution we have created in the matcher.
-    # We should have two paramspec expressions, lhs and rhs, matching the
-    # higher-order function's args and return value respectively.
-    rhs = return_callable.args[0]
-    if isinstance(rhs, pytd.Concatenate):
-      r_pspec = rhs.paramspec
-      r_args = rhs.args
-    else:
-      r_pspec = rhs
-      r_args = ()
-    if r_pspec.full_name not in subst:
-      # TODO(b/217789659): Should this be an assertion failure?
-      return
-    ret = self.ctx.program.NewVariable()
-    for data in subst[r_pspec.full_name].data:
-      sig = data.sig
-      ann = sig.annotations.copy()
-      ann["return"] = val
-      ret_posargs = []
-      for i, typ in enumerate(r_args):
-        name = f"_{i}"
-        ret_posargs.append(name)
-        if not _isinstance(typ, "BaseValue"):
-          typ = self.ctx.convert.constant_to_value(typ)
-        ann[name] = typ
-      # We have done prefix type matching in the matcher, so we can safely strip
-      # off the lhs args from the sig by count.
-      lhs = data.paramspec
-      l_nargs = len(lhs.args) if _isinstance(lhs, "Concatenate") else 0
-      param_names = tuple(ret_posargs) + sig.param_names[l_nargs:]
-      # All params need to be in the annotations dict or output.py crashes
-      for k in param_names:
-        if k and k not in ann:
-          ann[k] = self.ctx.convert.unsolvable
-      if sig.varargs_name and sig.varargs_name not in ann:
-        ann[sig.varargs_name] = self.ctx.convert.tuple_type
-      if sig.kwargs_name and sig.kwargs_name not in ann:
-        ann[sig.kwargs_name] = self.ctx.convert.dict_type
-      posonly_count = sig.posonly_count + len(r_args) - l_nargs
-      ret_sig = sig._replace(param_names=param_names, annotations=ann,
-                             posonly_count=posonly_count)
-      ret.AddBinding(_function_base.SimpleFunction(ret_sig, self.ctx))
-    ret_map[key] = ret
+    ret = self._paramspec_signature(return_callable, val, subst)
+    if ret:
+      ret_map[key] = ret
 
   def call_with_args(self, node, func, arg_dict, match, ret_map):
     """Call this signature. Used by PyTDFunction."""
@@ -708,15 +697,17 @@ class PyTDSignature(utils.ContextWeakrefMixin):
 
   @classmethod
   def _collect_mutated_parameters(cls, typ, mutated_type):
+    if not mutated_type:
+      return []
     if (isinstance(typ, pytd.UnionType) and
         isinstance(mutated_type, pytd.UnionType)):
       if len(typ.type_list) != len(mutated_type.type_list):
         raise ValueError(
             "Type list lengths do not match:\nOld: %s\nNew: %s" %
             (typ.type_list, mutated_type.type_list))
-      return itertools.chain.from_iterable(
+      return list(itertools.chain.from_iterable(
           cls._collect_mutated_parameters(t1, t2)
-          for t1, t2 in zip(typ.type_list, mutated_type.type_list))
+          for t1, t2 in zip(typ.type_list, mutated_type.type_list)))
     if typ == mutated_type and isinstance(typ, pytd.ClassType):
       return []  # no mutation needed
     if (not isinstance(typ, pytd.GenericType) or
@@ -724,7 +715,8 @@ class PyTDSignature(utils.ContextWeakrefMixin):
         typ.base_type != mutated_type.base_type or
         not isinstance(typ.base_type, pytd.ClassType)):
       raise ValueError(f"Unsupported mutation:\n{typ!r} ->\n{mutated_type!r}")
-    return [zip(mutated_type.base_type.cls.template, mutated_type.parameters)]
+    return [list(zip(mutated_type.base_type.cls.template,
+                     mutated_type.parameters))]
 
   def _get_mutation(self, node, arg_dict, subst, retvar):
     """Mutation for changing the type parameters of mutable arguments.
@@ -761,16 +753,7 @@ class PyTDSignature(utils.ContextWeakrefMixin):
       args = actual.data
       for arg in args:
         if isinstance(arg, _instance_base.SimpleValue):
-          try:
-            all_names_actuals = self._collect_mutated_parameters(
-                formal.type, formal.mutated_type)
-          except ValueError as e:
-            log.error("Old: %s", pytd_utils.Print(formal.type))
-            log.error("New: %s", pytd_utils.Print(formal.mutated_type))
-            log.error("Actual: %r", actual)
-            raise ValueError("Mutable parameters setting a type to a "
-                             "different base type is not allowed.") from e
-          for names_actuals in all_names_actuals:
+          for names_actuals in self.mutated_type_parameters[formal]:
             for tparam, type_actual in names_actuals:
               log.info("Mutating %s to %s",
                        tparam.name,
