@@ -78,6 +78,8 @@ Unsolvable = _singletons.Unsolvable
 AnnotationClass = _typing.AnnotationClass
 AnnotationContainer = _typing.AnnotationContainer
 ParamSpec = _typing.ParamSpec
+ParamSpecArgs = _typing.ParamSpecArgs
+ParamSpecKwargs = _typing.ParamSpecKwargs
 ParamSpecInstance = _typing.ParamSpecInstance
 Concatenate = _typing.Concatenate
 TypeParameter = _typing.TypeParameter
